/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorization

import (
	"encoding/json"
	"errors"
	"net/url"

	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// QueryAuthParam 获取授权信息参数
type QueryAuthParam struct {
	ComponentAppId    string `json:"component_appid"`
	AuthorizationCode string `json:"authorization_code"`
}

// QueryAuthResult 获取授权信息结果
type QueryAuthResult struct {
	wxerr.Error
	AuthorizationInfo AuthorizationInfo `json:"authorization_info"`
}

const (
	apiQueryAuth = "https://apies.weixin.qq.com/cgi-bin/component/api_query_auth"
)

// QueryAuth 获取授权信息
func QueryAuth(componentAccessToken string, data QueryAuthParam) (*QueryAuthResult, error) {
	if componentAccessToken == "" {
		return nil, errors.New("获取授权信息 第三方平台的 component_access_token 不能为空")
	}

	if data.ComponentAppId == "" {
		return nil, errors.New("获取授权信息 第三方平台的 appid 不能为空")
	}

	if data.AuthorizationCode == "" {
		return nil, errors.New("获取授权信息 授权码 不能为空")
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", componentAccessToken)

	apiUrl, _ := request.ParseURL(apiQueryAuth, &queryParam)
	resp, e2 := request.PostJSON(apiUrl.String(), data)
	if e2 != nil {
		return nil, e2
	}

	result := QueryAuthResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	if result.Code != 0 {
		return &result, result.Error
	}

	return &result, nil
}
