/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorizer

import (
	"encoding/json"
	"errors"
	"net/url"

	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// SetAuthorizerOptionParam 设置授权方选项信息参数
type SetAuthorizerOptionParam struct {
	ComponentAppId  string `json:"component_appid"`
	AuthorizerAppId string `json:"authorizer_appid"`
	OptionName      string `json:"option_name"`
	OptionValue     string `json:"option_value"`
}

// SetAuthorizerOptionResult 设置授权方选项信息结果
type SetAuthorizerOptionResult struct {
	wxerr.Error
}

const (
	apiSetAuthorizerOption = "https://apies.weixin.qq.com/cgi-bin/component/api_set_authorizer_option"
)

// SetAuthorizerOption 设置授权方选项信息
func SetAuthorizerOption(componentAccessToken string, data SetAuthorizerOptionParam) (*SetAuthorizerOptionResult, error) {
	if componentAccessToken == "" {
		return nil, errors.New("设置授权方选项信息 第三方平台的 component_access_token 不能为空")
	}

	if data.ComponentAppId == "" {
		return nil, errors.New("设置授权方选项信息 授权公众号或小程序的 appid 不能为空")
	}

	if data.AuthorizerAppId == "" {
		return nil, errors.New("设置授权方选项信息 第三方平台的 appid 不能为空")
	}

	if data.OptionName == "" {
		return nil, errors.New("设置授权方选项信息 选项名称 不能为空")
	}

	if data.OptionValue == "" {
		return nil, errors.New("设置授权方选项信息 设置的选项值 不能为空")
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", componentAccessToken)

	apiUrl, _ := request.ParseURL(apiSetAuthorizerOption, &queryParam)
	resp, e2 := request.PostJSON(apiUrl.String(), data)
	if e2 != nil {
		return nil, e2
	}

	result := SetAuthorizerOptionResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	if result.Code != 0 {
		return &result, result.Error
	}

	return &result, nil
}
